@@ -43,42 +43,42 @@ class IssueBoundary:
         self.name = name
 
     @staticmethod
-    def would_cause_lowercase_first_letter(span: Span) -> bool:
+    def _would_cause_lowercase_first_letter(span: Span) -> bool:
         return (
                 span.start_char == 0
                 and len(span._.suggestions) > 0
-                and "" in span._.suggestions[0]
+                and span._.suggestions[0] == ""
         )
 
     @staticmethod
-    def would_cause_double_space(span: Span) -> bool:
+    def _would_cause_double_space(span: Span) -> bool:
         return (
                 span.start_char > 0
                 and span.end_char < len(span.doc.text)
                 and len(span._.suggestions) > 0
                 and span.doc.text[span.start_char - 1] == " "
                 and span.doc.text[span.end_char] == " "
-                and any(item in ["", ","] for item in span._.suggestions[0])
+                and span._.suggestions[0] in ["", ","]
         )
 
     @staticmethod
-    def would_cause_space_at_start(span: Span) -> bool:
+    def _would_cause_space_at_start(span: Span) -> bool:
         return (
                 span.start_char == 0
                 and len(span._.suggestions) > 0
-                and "" in span._.suggestions[0]
+                and span._.suggestions[0] == ""
         )
 
     @staticmethod
-    def is_comma_replacement_that_could_be_deletion(span: Span) -> bool:
+    def _is_comma_replacement_that_could_be_deletion(span: Span) -> bool:
         return (
                 (span.text[0] == "," or span.text[-1] == ",")
                 and len(span._.suggestions) > 0
-                and "," in span._.suggestions[0]
+                and span._.suggestions[0] == ","
         )
 
     @staticmethod
-    def comma_replacement_to_deletion(span: Span):
+    def _comma_replacement_to_deletion(span: Span):
         """
         rather than suggest replacing 'some long string,' with ',', it looks nicer in the front end
         if we replace 'some long string' with '' - because the empty string is a magic replacement
@@ -98,44 +98,62 @@ class IssueBoundary:
             span.end_char -= 1
         span._.suggestions.remove(",")
         if "" not in span._.suggestions:
-            span._.suggestions.append("")
+            span._.suggestions.insert(0, "")
         return span
 
     @staticmethod
-    def would_cause_parenthesis_space(span: Span) -> bool:
+    def _would_cause_parenthesis_space(span: Span) -> bool:
         return (
                 span.start_char > 0
                 and span.end_char < len(span.doc.text)
                 and len(span._.suggestions) > 0
                 and span.doc.text[span.start_char - 1] == "("
                 and span.doc.text[span.end_char] == " "
-                and "" in span._.suggestions[0]
+                and span._.suggestions[0] in [""]
         )
 
     @staticmethod
-    def would_cause_space_parenthesis(span: Span) -> bool:
+    def _would_cause_space_parenthesis(span: Span) -> bool:
         return (
                 span.start_char > 0
                 and span.end_char < len(span.doc.text)
                 and len(span._.suggestions) > 0
                 and span.doc.text[span.start_char - 1] == " "
                 and span.doc.text[span.end_char] == ")"
-                and "" in span._.suggestions[0]
+                and span._.suggestions[0] in [""]
         )
 
+    @staticmethod
+    def _sanity_check(span: Span) -> Span:
+        if span[0].is_sent_start or span.doc.text[span.start_char - 1] == "(":
+            # the language model gets confused and sometimes thinks
+            # that it makes sense to start a sentence (or a parenthetical) with a comma
+            # don't allow this
+            if "," in span._.suggestions:
+                span._.suggestions.remove(",")
+        return span
+
+    @staticmethod
+    def _drop_comma_when_empty_top(span: Span) -> Span:
+        if len(span._.suggestions) > 0 and span._.suggestions[0] == "" and "," in span._.suggestions:
+            span._.suggestions.remove(",")
+        return span
+
     def __call__(self, spans: List[Span]) -> List[Span]:
         result = []
         for span in spans:
             span = ESpan.create_instance(span)
 
-            if IssueBoundary.would_cause_double_space(span):
+            span = IssueBoundary._sanity_check(span)
+
+            if IssueBoundary._would_cause_double_space(span):
                 # double space issue, extending issue back one character
                 span.start_char -= 1
-            elif IssueBoundary.would_cause_space_at_start(span):
+            elif IssueBoundary._would_cause_space_at_start(span):
                 # space at start, extending issue forward one character
                 span.end_char += 1
 
-            if IssueBoundary.would_cause_lowercase_first_letter(span):
+            if IssueBoundary._would_cause_lowercase_first_letter(span):
                 # casing issue, extending issue forward one word and uppercasing that word
                 doc_text_without_issue = span.doc.text[span.end_char:]
                 first_space_index = doc_text_without_issue.find(' ')
@@ -143,16 +161,19 @@ class IssueBoundary:
                 span.end_char += first_space_index
                 span._.suggestions = [replacement.capitalize()]
 
-            # order matters, must come after would_cause_lowercase_first_letter
-            if IssueBoundary.is_comma_replacement_that_could_be_deletion(span):
-                span = IssueBoundary.comma_replacement_to_deletion(span)
+            # order matters, must come after _would_cause_lowercase_first_letter
+            if IssueBoundary._is_comma_replacement_that_could_be_deletion(span):
+                span = IssueBoundary._comma_replacement_to_deletion(span)
 
             # fix parentheses
-            if self.would_cause_parenthesis_space(span):
+            if self._would_cause_parenthesis_space(span):
                 span.end_char += 1
-            if self.would_cause_space_parenthesis(span):
+            if self._would_cause_space_parenthesis(span):
                 span.start_char -= 1
 
+            # only "" or "," can ever be right; so drop comma suggestion, when first one is empty
+            span = IssueBoundary._drop_comma_when_empty_top(span)
+
             result.append(span)
 
         return sorted(result, key=lambda span: span.start)
